@@ -4,7 +4,11 @@
 {-# LANGUAGE TypeFamilies #-} -- For the IsList and IsString instances
 
 #if defined(__GLASGOW_HASKELL__) && __GLASGOW_HASKELL__ >= 708
-{-# LANGUAGE PatternSynonyms, ViewPatterns #-}
+{-# LANGUAGE PatternSynonyms #-}
+-- Mark this module as trustworthy even though we import 'IsList' from GHC.Exts,
+-- which is marked unsafe. 'IsList' is safe.
+{-# LANGUAGE Trustworthy #-}
+{-# LANGUAGE ViewPatterns #-}
 #endif
 
 -----------------------------------------------------------------------------
@@ -83,8 +87,8 @@ import Text.Read (Lexeme(Ident), lexP, parens, prec, readPrec, readListPrec,
 
 #if __GLASGOW_HASKELL__ >= 708
 import GHC.Exts (IsList)
--- This is for the IsList methods, which conflict with fromList, toList:
-import qualified GHC.Exts
+-- Make IsList type and methods visible for instance.
+import qualified GHC.Exts (IsList(Item, fromList, toList))
 #endif
 
 #endif
